package com.duanledexianxianxian.demo.redis.helper;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;

import java.util.List;
import java.util.Map;

import static com.duanledexianxianxian.demo.redis.config.RedisConfiguration.BEAN_NAME_CHECK_AND_SET_SCRIPT;
import static java.util.Arrays.asList;

/**
 * @author duanledexianxianxian
 */
public class RedisHelper {
    private final StringRedisTemplate StringRedisTemplate;
    private final RedisScript<String> checkAndSetScript;


    private <T> T carriedLuaScript(Class resultCls, String script, List<String> keys, Object... args) {
        DefaultRedisScript<T> redisScript = new DefaultRedisScript<>(script, resultCls);
        return StringRedisTemplate.execute(redisScript, keys, args);
    }

    private RedisHelper() {
        // 	It is ideal to configure a single instance of DefaultRedisScript in your application context to avoid re-calculation of the script’s SHA1 on every script run.
        StringRedisTemplate = SpringContextHelper.getBean(StringRedisTemplate.class);
        checkAndSetScript = SpringContextHelper.getBean(BEAN_NAME_CHECK_AND_SET_SCRIPT);
    }

    public static RedisHelper getInstance() {
        return RedisHelperInstance.INSTANCE;
    }

    private static class RedisHelperInstance {
        /**
         * The constant INSTANCE.
         */
        private static final RedisHelper INSTANCE = new RedisHelper();
    }


    /**
     * 批量检查某个数值并且减去相应值
     * <p>
     * todo 现在参数类型写死的，需使用泛型进行通用处理
     *
     * @param paramsMap
     * @return
     */
    public Map<String, String> batchCheckNum(Map<String, String> paramsMap) {
        if (paramsMap.isEmpty()) {
            return null;
        }

        // 将map转成list
        List<String> keys = Lists.newArrayList(paramsMap.keySet());
        List<String> values = Lists.newArrayList(paramsMap.values());

        String result = StringRedisTemplate.execute(checkAndSetScript, keys, values.toArray());
        if (StringUtils.isNotBlank(result)) {
            // todo 此处优化
            return JSONObject.parseObject(result,new TypeReference<Map<String,String>>() {});
        }
        return Maps.newHashMap();

    }
}
