package com.duanledexianxianxian.demo.redis.helper;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Locale;

/**
 * spring上下文工具类
 *
 * @author duanledexianxianxian
 */
@Component
public class SpringContextHelper implements ApplicationContextAware {


    /**
     * The constant invocation.
     */
    private static ApplicationContext context = null;
    /**
     * 是否为开发模式,方便在代码中获取进行判断
     */
    private static boolean isDevModel = false;

    public static boolean isDevModel() {
        return isDevModel;
    }

    public static void setIsDevModel(boolean isDevModel) {
        SpringContextHelper.isDevModel = isDevModel;
    }


    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        context = applicationContext;
    }

    /**
     * Gets bean.
     * 线程中可以使用
     *
     * @param <T>      the type parameter
     * @param beanName the bean name
     * @return the bean
     */
    public static <T> T getBean(String beanName) {
        return (T) context.getBean(beanName);
    }


    /**
     * Gets bean.
     *
     * @param <T>       the type parameter
     * @param className the class name
     * @return the bean
     */
    public static <T> T getBean(Class<T> className) {
        return context.getBean(className);
    }

    /**
     * Gets message.
     *
     * @param key the key
     * @return the message
     */
    public static String getMessage(String key) {
        Locale locale = LocaleContextHolder.getLocale();
        return context.getMessage(key, new Object[]{}, key, locale);
    }

    /**
     * Gets message.
     * 国际化使用
     *
     * @param key    the key
     * @param locale the locale
     * @return the message
     */
    public static String getMessage(String key, Locale locale) {
        return getMessage(key, new Object[]{}, locale);
    }


    /**
     * Gets message.
     *
     * @param key   the key
     * @param param the param
     * @return the message
     */
    public static String getMessage(String key, Object[] param) {
        Locale locale = LocaleContextHolder.getLocale();
        return context.getMessage(key, param, key, locale);
    }

    /**
     * Gets message.
     *
     * @param key    the key
     * @param param  the param
     * @param locale the locale
     * @return the message
     */
    public static String getMessage(String key, Object[] param, Locale locale) {
        return context.getMessage(key, param, key, locale);
    }


    /**
     * 获取当前环境
     *
     * @return the active message
     */
    public static String getActiveProfile() {
        return context.getEnvironment().getActiveProfiles()[0];
    }


}
