package com.duanledexianxianxian.demo.redis.service;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.duanledexianxianxian.demo.redis.helper.RedisHelper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * redis相关测试服务类
 *
 * @author duanledexianxianxian
 */
@Service
public class RedisService {
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    private <T> T carriedLuaScript(Class resultCls, String script, List<String> keys, Object... args) {
        DefaultRedisScript<T> redisScript = new DefaultRedisScript<>(script, resultCls);
        return redisTemplate.execute(redisScript, keys, args);
    }


    /**
     * 传说中的狂把子 hello world
     *
     * @return string string
     */
    public String testRedis1() {
        return carriedLuaScript(String.class, "return 'Hello Redis' ", null);
    }


    /**
     * 测试有参数的情况
     * <p>
     * 在redis中新建了stock的key为： stock:1=20
     *
     * @param num the num
     * @return long long
     */
    public Long testRedis2(Long num) {
        String key = "stock:1";
        redisTemplate.opsForValue().set(key, "20");
        String script = "local stock=tonumber(redis.call('GET',KEYS[1]));" + "local num=tonumber(ARGV[1]);" + "if(stock >= num) then " + "return redis.call('incrby',KEYS[1],0-num);" + "end;" + "return -1;";
        return carriedLuaScript(Long.class, script, Collections.singletonList(key), String.valueOf(num));
    }


    /**
     * 批量操作
     *
     * @param paramsMap the params map
     * @return long string
     */
    public Map<String, Long> testRedis3(Map<String, String> paramsMap) {
        // 初始化数据
        Map<String, String> initDataMap = Maps.newHashMap();
        initDataMap.put("stock:1", "20");
        initDataMap.put("stock:2", "100");
        initDataMap.put("stock:3", "0");
        redisTemplate.opsForValue().multiSet(initDataMap);

        if (paramsMap.isEmpty()) {
            return null;
        }

        // 将map转成list
        List<String> keys = Lists.newArrayList(paramsMap.keySet());
        List<String> values = Lists.newArrayList(paramsMap.values());

        String script = "" + "local keys,values = KEYS,ARGV " + "local result = {s='ss'}; " + "for i, v in ipairs(keys) do " + "   local stock = tonumber(redis.call('GET', v));" + "   local num = tonumber(values[i]);" + "   if (stock >= num) then" + "       result[v]=redis.call('incrby', v, 0 - num);" + "   else " + "       result[v]=-1;" + "   end " + "end " + "redis.log(redis.LOG_NOTICE,'hello world') " + "for k,v in pairs(result) do\n" + "        redis.log(redis.LOG_NOTICE, k,result[k]) " + "end " + "return cjson.encode(result);";
        String result = carriedLuaScript(String.class, script, keys, values.toArray());
        Map<String, Long> map = JSONObject.parseObject(result, Map.class);
        return map;
    }


    /**
     * 调用封装之后的批量操作
     * 关于集群
     * 方法一：slot需要自己通过key控制
     * redis中只需要{}中值一样，则会分配到相同的slots
     * 方法二：使用一个batchId作为key value:{key1:value1,key2:value2...}
     * 方法很多
     *
     * @param paramsMap
     * @return
     */
    public Map<String, String> testRedis4(Map<String, String> paramsMap) {
        return RedisHelper.getInstance().batchCheckNum(paramsMap);
    }


}
